@@ -6,41 +6,22 @@
     structure(x, units = value, dim = dim, class = "units")
 }
 
-#' Unit creation
-#'
-#' A number of functions are provided for creating unit objects. 
-#' \itemize{
-#'     \item \code{as_units}, a generic with methods for a
-#'     character string and for quoted language. Note, direct usage of this function
-#'     by users is typically not necessary, as coercion via \code{as_units} is
-#'     automatically done with \code{`units<-`} and \code{set_units()}.
-#'    
-#'     \item \code{make_units()}, constructs units from bare expressions.
-#'     \code{make_units(m/s)} is equivalent to \code{as_units(quote(m/s))}
-#'    
-#'     \item \code{set_units()}, a pipe_friendly version of \code{`units<-`}. By
-#'     default it operates with bare expressions like \code{make_unit}, but this
-#'     behavior can be disabled by a specifying \code{mode = "standard"} or setting 
-#'     \code{units_options(set_units_mode = "standard")}.
-#' }
-#' 
+#' @name units
 #' @export
-#' @rdname as_units
-#' 
+#'
 #' @param bare_expression a bare R expression describing units. Must be valid R
 #'   syntax (reserved R syntax words like \code{in} must be backticked)
 #'
-#' @noMd
 #' @examples
-#' # The easiest way to assign units to a numeric vector is like this: 
+#' # The easiest way to assign units to a numeric vector is like this:
 #' x <- y <- 1:4
 #' units(x) <- "m/s"  # meters / second
-#' 
+#'
 #' # Alternatively, the easiest pipe-friendly way to set units:
 #' if(requireNamespace("magrittr", quietly = TRUE)) {
 #'   library(magrittr)
 #'   y %>% set_units(m/s)
-#' } 
+#' }
 #'
 #' # these are different ways of creating the same unit:
 #' # meters per second squared, i.e, acceleration
@@ -63,8 +44,8 @@
 #' }
 #' all_identical(x1, x2, x3, x4, x5, x6, x7, x8)
 #'
-#' # Note, direct usage of these unit creation functions is typically not 
-#' # necessary, since coercion is automatically done via as_units(). Again, 
+#' # Note, direct usage of these unit creation functions is typically not
+#' # necessary, since coercion is automatically done via as_units(). Again,
 #' # these are all equivalent ways to generate the same result.
 #'
 #' x1 <- x2 <- x3 <- x4 <- x5 <- x6 <- x7 <- x8 <- 1:4
@@ -109,24 +90,22 @@
 #' # For example, a microbiologist might work with counts of bacterial cells
 #' # make_units(cells/ml) # by default, throws an ERROR
 #' # First define the unit, then the newly defined unit is accepted.
-#' install_symbolic_unit("cells")
-#' make_units(cells/ml) 
-#' 
-#' # Note, install_symbolic_unit() does not add any support for unit
-#' # conversion, or arithmetic operations that require unit conversion. See
-#' # ?install_conversion_constant for defining relationships between user 
-#' # defined units.
+#' install_unit("cells")
+#' make_units(cells/ml)
+#'
+#' # Note that install_unit() adds support for defining relationships between
+#' # the newly created symbols or names and existing units.
 #'
 #' ## set_units()
-#' # set_units is a pipe friendly version of `units<-`. 
+#' # set_units is a pipe friendly version of `units<-`.
 #' if(requireNamespace("magrittr", quietly = TRUE)) {
 #'   library(magrittr)
 #'   1:5 %>% set_units(N/m^2)
 #'   # first sets to m, then converts to km
 #'   1:5 %>% set_units(m) %>% set_units(km)
 #' }
-#' 
-#' # set_units has two modes of operation. By default, it operates with 
+#'
+#' # set_units has two modes of operation. By default, it operates with
 #' # bare symbols to define the units.
 #' set_units(1:5, m/s)
 #'
@@ -135,10 +114,10 @@
 #' # thought of as a simple alias for `units<-` that is pipe friendly.
 #' set_units(1:5, "m/s", mode = "standard")
 #' set_units(1:5, make_units(m/s), mode = "standard")
-#' 
+#'
 #' # the mode of set_units() can be controlled via a global option
 #' # units_options(set_units_mode = "standard")
-#' 
+#'
 #' # To remove units use
 #' units(x) <- NULL
 #' # or
@@ -149,8 +128,70 @@ make_units <- function(bare_expression, check_is_valid = TRUE) {
   as_units.call(substitute(bare_expression), check_is_valid = check_is_valid)
 }
 
+#' @name units
+#' @export
+as_units <- function(x, ...) {
+  UseMethod("as_units")
+}
 
-#  ----- as_units.character helpers ------ 
+#' @name units
+#' @export
+as_units.default <- function(x, value = unitless, ...) {
+  if (is.null(x)) return(x)
+  units(x) <- value
+  x
+}
+
+#' @name units
+#' @export
+as_units.units <- function(x, value, ...) {
+  if(!missing(value) && !identical(units(value), units(x)))
+    warning("Use set_units() to perform unit conversion. Return unit unmodified")
+  x
+}
+
+#' @name units
+#' @export
+as_units.symbolic_units <- function(x, value, ...) {
+  if(!missing(value))
+    warning("supplied value ignored")
+  .as.units(1L, x)
+}
+
+#' @examples
+#' s = Sys.time()
+#' d  = s - (s+1)
+#' as_units(d)
+#'
+#' @name units
+#' @export
+as_units.difftime <- function(x, value, ...) {
+  u <- attr(x, "units")
+  x <- unclass(x)
+  attr(x, "units") <- NULL
+
+  # convert from difftime to udunits2:
+  if (u == "secs") # secs -> s
+    x <- x * symbolic_unit("s")
+  else if (u == "mins") # mins -> min
+    x <- x * symbolic_unit("min")
+  else if (u == "hours") # hours -> h
+    x <- x * symbolic_unit("h")
+  else if (u == "days") # days -> d
+    x <- x * symbolic_unit("d")
+  else if (u == "weeks") { # weeks -> 7 days
+    x <- 7 * x
+    x <- x * symbolic_unit("d")
+  } else
+    stop(paste("unknown time units", u, "in difftime object"))
+
+  if (!missing(value)) # convert optionally:
+    units(x) <- value
+
+  x
+}
+
+#  ----- as_units.character helpers ------
 
 backtick <- function(x) {
   # backtick all character runs uninterupted by one of ^()*^/`- or a space
@@ -161,7 +202,7 @@ backtick <- function(x) {
 
 are_exponents_implicit <- function(s) {
   s <- trimws(s)
-  has <- function(chr, regex = FALSE) 
+  has <- function(chr, regex = FALSE)
     grepl(chr, s, fixed = !regex, perl = regex)
   !has("^") && !has("*") && !has("/") && has("\\s|\\D.*\\d$", regex = TRUE)
 }
@@ -170,14 +211,12 @@ is_udunits_time <- function(s) {
   ud_is_parseable(s) && ud_are_convertible(s, "seconds since 1970-01-01")
 }
 
-
-#' @rdname as_units
+#' @name units
 #' @export
-#' @noMd
 #'
 #' @param force_single_symbol Whether to perform no string parsing and force
 #'   treatment of the string as a single symbol.
-#' 
+#'
 #' @param implicit_exponents If the unit string is in product power form (e.g.
 #'   \code{"km m-2 s-1"}). Defaults to \code{NULL}, in which case a guess is made
 #'   based on the supplied string. Set to \code{TRUE} or \code{FALSE} if the guess is
@@ -218,6 +257,12 @@ is_udunits_time <- function(s) {
 #'   symbol may optionally be followed by a single number, specifying the power.
 #'   For example \code{"m2 s-2"} is equivalent to \code{"(m^2)*(s^-2)"}.
 #'
+#'   It must be noted that prepended numbers are supported too, but their
+#'   interpretation slightly varies depending on whether they are separated from
+#'   the unit string or not. E.g., \code{"1000 m"} is interpreted as magnitude
+#'   and unit, but \code{"1000m"} is interpreted as a prefixed unit, and it is
+#'   equivalent to \code{"km"} to all effects.
+#'
 #'   The third type of unit string format accepted is the special case of
 #'   udunits time duration with a reference origin, for example \code{"hours
 #'   since 1970-01-01 00:00:00"}. Note, that the handling of time and calendar
@@ -226,25 +271,28 @@ is_udunits_time <- function(s) {
 #'   users that work with udunits time data, e.g., with NetCDF files. Users are
 #'   otherwise encouraged to use \code{R}'s date and time functionality provided
 #'   by \code{Date} and \code{POSIXt} classes.
-as_units.character <- function(x, 
+#'
+as_units.character <- function(x,
                                check_is_valid = TRUE,
-                               implicit_exponents = NULL, 
+                               implicit_exponents = NULL,
                                force_single_symbol = FALSE, ...) {
 
   stopifnot(is.character(x), length(x) == 1)
-  
+
+  if (isTRUE(x == "")) return(unitless)
+
   if(force_single_symbol || is_udunits_time(x))
     return(symbolic_unit(x, check_is_valid = check_is_valid))
-  
+
   if(is.null(implicit_exponents))
     implicit_exponents <- are_exponents_implicit(x)
-  
+
   if(implicit_exponents)
     x <- convert_implicit_to_explicit_exponents(x)
-  
+
   x <- backtick(x)
   o <- try(expr <- parse(text = x)[[1]], silent = TRUE)
-  
+
   if(inherits(o, "try-error")) {
     warning("Could not parse expression: ", sQuote(x),          # nocov
       ". Returning as a single symbolic unit()", call. = FALSE) # nocov
@@ -263,7 +311,7 @@ convert_implicit_to_explicit_exponents <- function(x) {
   x
 }
 
-#  ----- as_units.call helpers ------ 
+#  ----- as_units.call helpers ------
 
 # from package:yasp, paste collapse with serial (oxford) comma
 pc_and <- function(..., sep = "") {
@@ -282,38 +330,34 @@ pc_and <- function(..., sep = "") {
 #`%not_in%` <- function(x, table) match(x, table, nomatch = 0L) == 0L
 
 .msg_units_not_recognized <- function(unrecognized_symbols, full_expr) {
-    
+
   if (is.language(full_expr))
     full_expr <- deparse(full_expr)
-  
-  is_are <- if (length(unrecognized_symbols) > 1L) "are" else "is" 
-  
-  paste0("In ", sQuote(full_expr), ", ", 
+
+  is_are <- if (length(unrecognized_symbols) > 1L) "are" else "is"
+
+  paste0("In ", sQuote(full_expr), ", ",
     pc_and(sQuote(unrecognized_symbols)), " ", is_are, " not recognized by udunits.\n\n",
     "See a table of valid unit symbols and names with valid_udunits().\n",
-    "Custom user-defined units can be added with install_symbolic_unit().\n\n",
+    "Custom user-defined units can be added with install_unit().\n\n",
     "See a table of valid unit prefixes with valid_udunits_prefixes().\n",
     "Prefixes will automatically work with any user-defined unit.")
 }
 
-is_valid_unit_symbol <- function(chr) {
-  ud_is_parseable(chr)
-}
-
 units_eval_env <- new.env(parent = baseenv())
 units_eval_env$ln <- function(x) base::log(x)
 units_eval_env$lg <- function(x) base::log(x, base = 10)
 units_eval_env$lb <- function(x) base::log(x, base = 2)
 
 
+#' @name units
 #' @export
-#' @rdname as_units
 #'
 #' @param check_is_valid throw an error if all the unit symbols are not either
 #'   recognized by udunits2 via \code{ud_is_parseable()}, or a custom
-#'   user defined via \code{install_symbolic_unit()}. If \code{FALSE}, no check
+#'   user defined via \code{install_unit()}. If \code{FALSE}, no check
 #'   for validity is performed.
-#'   
+#'
 #' @note By default, unit names are automatically substituted with unit names
 #'   (e.g., kilogram --> kg). To turn off this behavior, set
 #'   \code{units_options(auto_convert_names_to_symbols = FALSE)}
@@ -323,139 +367,99 @@ units_eval_env$lb <- function(x) base::log(x, base = 2)
 #'   In \code{as_units()}, each of the symbols in the unit expression is treated
 #'   individually, such that each symbol must be recognized by the udunits
 #'   database (checked by \code{ud_is_parseable()}, \emph{or} be a custom,
-#'   user-defined unit symbol that was defined either by
-#'   \code{install_symbolic_unit()} or \code{install_conversion_constant()}. To
+#'   user-defined unit symbol that was defined by \code{install_unit()}. To
 #'   see which symbols and names are currently recognized by the udunits
-#'   database, see \code{udunits_symbols()}.
+#'   database, see \code{valid_udunits()}.
 #'
-#' @return A new unit object that can be used in arithmetic, unit conversion or
-#'   unit assignment.
-#'
-#' @seealso \code{\link{valid_udunits}}
+#' @seealso \code{\link{install_unit}}, \code{\link{valid_udunits}}
 as_units.call <- function(x, check_is_valid = TRUE, ...) {
-  
-  if(missing(x) || identical(x, quote(expr =)) || 
+
+  if(missing(x) || identical(x, quote(expr =)) ||
      identical(x, 1) || identical(x, 1L))
     return(.as.units(1, unitless))
-  
+
   if (is.vector(x) && any(is.na(x)))
   	stop("a missing value for units is not allowed")
 
   stopifnot(is.language(x))
-  
+
   vars <- all.vars(x)
   if(!length(vars))
     stop(call. = FALSE,
 "No symbols found. Please supply bare expressions with this approach.
 See ?as_units for usage examples.")
-  
+
   if (check_is_valid) {
-    valid <- vapply(vars, is_valid_unit_symbol, logical(1L))
+    valid <- vapply(vars, ud_is_parseable, logical(1L))
     if (!all(valid))
       stop(.msg_units_not_recognized(vars[!valid], x), call. = FALSE)
   }
-  
+
   names(vars) <- vars
   tmp_env <- lapply(vars, symbolic_unit, check_is_valid = FALSE)
- 
+
   if (dont_simplify_here <- is.na(.units.simplify())) {
   	units_options(simplify = FALSE)
   	on.exit(units_options(simplify = NA))
   }
-  
+
   unit <- tryCatch( eval(x, tmp_env, units_eval_env),
     error = function(e) stop( paste0( conditionMessage(e), "\n",
           "Did you try to supply a value in a context where a bare expression was expected?"
         ), call. = FALSE ))
 
-#  if(as.numeric(unit) %not_in% c(1, 0)) # 0 if log() used. 
+#  if(as.numeric(unit) %not_in% c(1, 0)) # 0 if log() used.
 #    stop(call. = FALSE,
-#"In ", sQuote(deparse(x)), " the numeric multiplier ", sQuote(as.numeric(unit)), " is invalid. 
-#Use `install_conversion_constant()` to define a new unit that is a multiple of another unit.")
-  
+#"In ", sQuote(deparse(x)), " the numeric multiplier ", sQuote(as.numeric(unit)), " is invalid.
+#Use `install_unit()` to define a new unit that is a multiple of another unit.")
+
   .as.units(as.numeric(unit), units(unit))
 }
 
-
+#' @name units
 #' @export
 as_units.expression <- as_units.call
 
+#' @name units
 #' @export
 as_units.name       <- as_units.call
 
+#' @name units
+#' @export
+as_units.POSIXt = function(x, value, ...) {
+  u = as.numeric(as.POSIXct(x))
+  units(u) = symbolic_unit("seconds since 1970-01-01 00:00:00 +00:00")
+  if (! missing(value))
+    units(u) = symbolic_unit(value)
+  u
+}
+
+#' @name units
+#' @export
+as_units.Date = function(x, value, ...) {
+  u = as.numeric(x)
+  units(u) = symbolic_unit("days since 1970-01-01")
+  if (!missing(value))
+    units(u) = symbolic_unit(value)
+  u
+}
+
 
 symbolic_unit <- function(chr, check_is_valid = TRUE) {
-  
+
   stopifnot(is.character(chr), length(chr) == 1)
-  
-  if (check_is_valid && !is_valid_unit_symbol(chr)) {
+
+  if (check_is_valid && !ud_is_parseable(chr)) {
     msg <- paste(sQuote(chr), "is not a unit recognized by udunits or a user-defined unit")
     stop(msg, call. = FALSE)
   }
-  
+
   auto_convert <- units_options("auto_convert_names_to_symbols")
   if (auto_convert && ud_is_parseable(chr)) {
     sym <- ud_get_symbol(chr)
-    if (nzchar(sym)) 
+    if (length(sym))
       chr <- sym
   }
-  
+
   .as.units(1, .symbolic_units(chr))
 }
-
-
-#' Drop Units
-#' 
-#' Drop units attribute and class.
-#' 
-#' @param x an object with units metadata.
-#' 
-#' @return the numeric without any units attributes, while preserving other
-#' attributes like dimensions or other classes.
-#'   
-#' @details Equivalent to \code{units(x) <- NULL}, or the pipe-friendly version
-#' \code{set_units(x, NULL)}, but \code{drop_units} will fail if the object has
-#' no units metadata. Use the alternatives if you want this operation to succeed
-#' regardless of the object type.
-#' 
-#' A \code{data.frame} method is also provided, which checks every column and
-#' drops units if any.
-#' 
-#' @export
-#' @examples
-#' x <- 1
-#' y <- set_units(x, m/s)
-#' 
-#' # this succeeds
-#' drop_units(y)
-#' set_units(y, NULL)
-#' set_units(x, NULL)
-#' 
-#' \dontrun{
-#' # this fails
-#' drop_units(x)
-#' }
-#' 
-#' df <- data.frame(x=x, y=y)
-#' df
-#' drop_units(df)
-#' 
-drop_units <- function(x) UseMethod("drop_units")
-
-#' @name drop_units
-#' @export
-drop_units.units <- function(x) {
-  class(x) <- setdiff(class(x), "units")
-  attr(x, "units") <- NULL
-  x
-}
-
-#' @name drop_units
-#' @export
-drop_units.data.frame <- function(x) {
-  for (i in seq_along(x)) {
-    if (inherits(x[[i]], "units"))
-      x[[i]] <- drop_units(x[[i]])
-  }
-  x
-}
